######################
###                ###
###   Exercise 1   ###
###                ###
######################
	### Explorations with part-of-speech tagged corpora. 
	### Tokenize the Brown Corpus and build one or more suitable 
	### data structures so that you can answer the following questions.

### 1. Read the Brown corpus
from nltk.corpus import brown
from nltk.probability import FreqDist
from nltk.probability import ConditionalFreqDist

### 2. Structures we need for quick answer to the questions
wordTagDist = ConditionalFreqDist() ### (word,tag) frequencies
tagDist     = FreqDist()            ### tag frequencies

### 3. Read the Brown corpus and populate the structures
for item in brown.items():
	text_token = brown.read(item)
	for token in text_token['WORDS']:
		tag = token['TAG']
		lowerToken = token['TEXT'].lower()	# normalize the word (i.e. convert to LOWERCASE!)
		wordTagDist[lowerToken].inc(tag)
		tagDist.inc(tag)


##########################
###   Exercise 1 (a)   ###
##########################
	### What is the most frequent tag? (This is the tag we would want to assign with a DefaultTagger)
print '\n(1a) The most frequent tag is:', tagDist.max()
print tagDist


##########################
###   Exercise 1 (b)   ###
##########################
	### Which word has the greatest number of distinct tags?
maxTagsCnt = -1
maxTagsWord = 'N/A'
for word in wordTagDist.conditions():
	if wordTagDist[word].B() > maxTagsCnt:
		maxTagsCnt = wordTagDist[word].B()
		maxTagsWord = word
print '\n(1b) The word with the greatest number of distinct tags is:', maxTagsWord


##########################
###   Exercise 1 (c)   ###
##########################
	### What is the ratio of masculine to feminine pronouns?
mascFemRatioHeSheOnly = 1.0 * wordTagDist['he'].count('pps') / wordTagDist['she'].count('pps')
print '\n(1c) The the ratio of masculine to feminine pronouns (he/she only!) is:', mascFemRatioHeSheOnly

masc = wordTagDist['his'].count('pp$$') + wordTagDist['himself'].count('ppl') +\
       wordTagDist['him'].count('ppo') + wordTagDist['he'].count('pps') +\
       wordTagDist['he\'s'].count('pps+bez') + wordTagDist['he\'d'].count('pps+hvd') +\
       wordTagDist['he\'s'].count('pps+hvz') + wordTagDist['he\'ll'].count('pps+md') +\
       wordTagDist['he\'d'].count('pps+md')
fem  = wordTagDist['hers'].count('pp$$') + wordTagDist['herself'].count('ppl') +\
       wordTagDist['her'].count('ppo') + wordTagDist['she'].count('pps') +\
       wordTagDist['she\'s'].count('pps+bez') + wordTagDist['she\'d'].count('pps+hvd') +\
       wordTagDist['she\'s'].count('pps+hvz') + wordTagDist['she\'ll'].count('pps+md') +\
       wordTagDist['she\'d'].count('pps+md')
mascFemRatio = 1.0 * masc / fem
print '(1c) The the ratio of masculine to feminine pronouns is:', mascFemRatio


##########################
###   Exercise 1 (d)   ###
##########################
	### How many words are ambiguous, in the sense that they appear with at least two tags?
totalWordTypesCnt  = 0
ambigWordTypesCnt  = 0
for word in wordTagDist.conditions():
	totalWordTypesCnt = totalWordTypesCnt + 1
	if wordTagDist[word].B() > 1:
		ambigWordTypesCnt = ambigWordTypesCnt + 1
print '\n(1d) Number of ambiguous word types:', ambigWordTypesCnt
print '(1d) Number of all word types (vocabulary size):', totalWordTypesCnt
print '(1d) Percentage of ambiguous vocabulary types: %0.2f%%' % (100.0 * ambigWordTypesCnt / totalWordTypesCnt)


##########################
###   Exercise 1 (e)   ###
##########################
	### What percentage of word occurrences in the Brown Corpus involve these ambiguous words?
totalWordTokensCnt  = 0
ambigWordTokensCnt  = 0
for word in wordTagDist.conditions():
	totalWordTokensCnt = totalWordTokensCnt + wordTagDist[word].N()
	if wordTagDist[word].B() > 1:
		ambigWordTokensCnt = ambigWordTokensCnt + wordTagDist[word].N()
print '\n(1e) Percentage of word occurrences in the Brown Corpus involving ambiguous words: %0.2f%%' %\
      (100.0 * ambigWordTokensCnt / totalWordTokensCnt)


##########################
###   Exercise 1 (f)   ###
##########################
	### Which nouns are more common in their plural form than their singular form? (Only consider regular plurals, formed with the -s suffix.)
### SINGULAR NOUNS: nn np nr (nn$ nn+bez nn+hvz np$ np+bez nr$)
### PLURAL NOUNS:   nns nps nrs (nns$ nps$)
wordListNN = []
wordListNP = []
wordListNR = []
for word in wordTagDist.conditions():
	if (wordTagDist[word].count('nn') > 0)\
	and (wordTagDist[word + 's'].count('nns') > 0)\
	and (wordTagDist[word].count('nn') < wordTagDist[word + 's'].count('nns') > 0):
		wordListNN.append((word, wordTagDist[word].count('nn'), wordTagDist[word + 's'].count('nns')))
	if (wordTagDist[word].count('np') > 0)\
	and (wordTagDist[word + 's'].count('nps') > 0)\
	and (wordTagDist[word].count('np') < wordTagDist[word + 's'].count('nps') > 0):
		wordListNP.append((word, wordTagDist[word].count('np'), wordTagDist[word + 's'].count('nps')))
	if (wordTagDist[word].count('nr') > 0)\
	and (wordTagDist[word + 's'].count('nrs') > 0)\
	and (wordTagDist[word].count('nr') < wordTagDist[word + 's'].count('nrs') > 0):
		wordListNR.append((word, wordTagDist[word].count('nr'), wordTagDist[word + 's'].count('nrs')))
print '\n(1f) Nouns which are more common in their plural form than their singular form'
print 'NN & NNS:', wordListNN
print 'NP & NPS:', wordListNP
print 'NR & NRS:', wordListNR


##########################
###   Exercise 1 (g)   ###
##########################
	### Produce an alphabetically sorted list of the distinct words tagged as md. 
mdList = []
for word in wordTagDist.conditions():
	if wordTagDist[word].count('md') > 0:
		mdList.append( (word, wordTagDist[word].count('md')) )
mdList.sort()
print '\n(1g) Here is an alphabetically sorted list of the distinct words tagged as md:'
print mdList


##########################
###   Exercise 1 (h)   ###
##########################
	### Identify words which can be plural nouns or third person singular verbs (e.g. deals). 
### noun plural:              nns nps nrs
### 3rd person singular verb: vbz
wordListH = []
for word in wordTagDist.conditions():
	nnCnt  = wordTagDist[word].count('nns') + wordTagDist[word].count('nps') + wordTagDist[word].count('nrs')
	vbzCnt = wordTagDist[word].count('vbz')
	if (nnCnt > 0) and (vbzCnt > 0):
		wordListH.append(word)
print '\n(1h) Identify words which can be plural nouns or third person singular verbs:'
print wordListH
