######################
###                ###
###   Exercise 5   ###
###                ###
######################
	### Combining taggers with BackoffTagger. There is typically a trade-off
	### between the accuracy and coverage for taggers: taggers that use more
	### specific contexts usually produce more accurate results, when they
	### have seen those contexts in the training data; but because the
	### training data is limited, they are less likely to encounter each
	### context. The BackoffTagger addresses this problem by trying taggers
	### with more specific contexts first; and falling back to the more general
	### taggers when necessary. In this exercise, we examine the effects of
	### using BackoffTagger. Create a DefaultTagger or a RegexpTagger, and
	### a UnigramTagger, and a NthOrderTagger. Train the UnigramTagger and
	### the NthOrderTagger using part of the Brown corpus. 


### 1. Read Brown corpus
from nltk.corpus import brown
from nltk.tagger import *
train_tokens = []
for item in brown.items()[:10]:    # texts 0-9
	train_tokens.append(brown.read(item))

test_tokens = []
for item in brown.items()[10:12]:    # texts 10-11
	test_tokens.append(brown.read(item))


##########################
###   Exercise 5 (a)   ###
##########################
	### Evaluate each tagger on unseen data from the Brown corpus.
	### Record the accuracy of the tagger (the percentage of tokens
	### that are correctly tagged). Be sure to use a different section
	### of the corpus for testing than you used for training. 

# Train the taggers
subtagger1 = RegexpTagger([(r'^[0-9]+(.[0-9]+)?$', 'cd'), (r'.*', 'nn')], SUBTOKENS='WORDS')
subtagger2 = UnigramTagger(SUBTOKENS='WORDS')           # zeroth order tagger
subtagger3 = NthOrderTagger(1, SUBTOKENS='WORDS')       # first order tagger
subtagger4 = NthOrderTagger(2, SUBTOKENS='WORDS')       # second order tagger
subtagger5 = NthOrderTagger(3, SUBTOKENS='WORDS')       # third order tagger
subtagger6 = NthOrderTagger(4, SUBTOKENS='WORDS')       # fourth order tagger

# Train the taggers
for tok in train_tokens:
	subtagger2.train(tok)
	subtagger3.train(tok)
	subtagger4.train(tok)
	subtagger5.train(tok)
	subtagger6.train(tok)
	
# Output the training accuracy results
print '\n(5a) TRAINING accuracy:'
print 'RegExp tagger  : %5.2f%%' % (100.0 * tagger_accuracy(subtagger1, train_tokens))
print 'Unigram tagger : %5.2f%%' % (100.0 * tagger_accuracy(subtagger2, train_tokens))
print 'Bigram tagger  : %5.2f%%' % (100.0 * tagger_accuracy(subtagger3, train_tokens))
print 'Trigram tagger : %5.2f%%' % (100.0 * tagger_accuracy(subtagger4, train_tokens))
print 'Fourgram tagger: %5.2f%%' % (100.0 * tagger_accuracy(subtagger5, train_tokens))
print 'Fivegram tagger: %5.2f%%' % (100.0 * tagger_accuracy(subtagger6, train_tokens))

print '\n(5a) TESTING accuracy:'
print 'RegExp tagger  : %5.2f%%' % (100.0 * tagger_accuracy(subtagger1, test_tokens))
print 'Unigram tagger : %5.2f%%' % (100.0 * tagger_accuracy(subtagger2, test_tokens))
print 'Bigram tagger  : %5.2f%%' % (100.0 * tagger_accuracy(subtagger3, test_tokens))
print 'Trigram tagger : %5.2f%%' % (100.0 * tagger_accuracy(subtagger4, test_tokens))
print 'Fourgram tagger: %5.2f%%' % (100.0 * tagger_accuracy(subtagger5, test_tokens))
print 'Fivegram tagger: %5.2f%%' % (100.0 * tagger_accuracy(subtagger6, test_tokens))


##########################
###   Exercise 5 (b)   ###
##########################
	### Use BackoffTagger to create three different combinations of
	### the basic taggers. Test the accuracy of each combined tagger.
	### Which combinations give the most improvement? 

# Combine the taggers
tagger1 = BackoffTagger([subtagger1], SUBTOKENS='WORDS')
tagger2 = BackoffTagger([subtagger2, subtagger1], SUBTOKENS='WORDS')
tagger3 = BackoffTagger([subtagger3, subtagger2, subtagger1], SUBTOKENS='WORDS')
tagger4 = BackoffTagger([subtagger4, subtagger3, subtagger2, subtagger1], SUBTOKENS='WORDS')
tagger5 = BackoffTagger([subtagger5, subtagger4, subtagger3, subtagger2, subtagger1], SUBTOKENS='WORDS')
tagger6 = BackoffTagger([subtagger6, subtagger5, subtagger4, subtagger3, subtagger2, subtagger1], SUBTOKENS='WORDS')

# Output the training accuracy results
print '\n(5b) TRAINING accuracy:'
print 'RegExp tagger                                          : %5.2f%%' % (100.0 * tagger_accuracy(tagger1, train_tokens))
print 'Unigram+RegExp tagger                                  : %5.2f%%' % (100.0 * tagger_accuracy(tagger2, train_tokens))
print 'Bigram+Unigram+RegExp tagger                           : %5.2f%%' % (100.0 * tagger_accuracy(tagger3, train_tokens))
print 'Trigram+Bigram+Unigram+RegExp tagger                   : %5.2f%%' % (100.0 * tagger_accuracy(tagger4, train_tokens))
print 'Fourgram+Trigram+Bigram+Unigram+RegExp tagger          : %5.2f%%' % (100.0 * tagger_accuracy(tagger5, train_tokens))
print 'Fivegram+Fourgram+Trigram+Bigram+Unigram+RegExp tagger : %5.2f%%' % (100.0 * tagger_accuracy(tagger6, train_tokens))

# Output the training accuracy results
print '\n(5b) TESTING accuracy:'
print 'RegExp tagger                                          : %5.2f%%' % (100.0 * tagger_accuracy(tagger1, test_tokens))
print 'Unigram+RegExp tagger                                  : %5.2f%%' % (100.0 * tagger_accuracy(tagger2, test_tokens))
print 'Bigram+Unigram+RegExp tagger                           : %5.2f%%' % (100.0 * tagger_accuracy(tagger3, test_tokens))
print 'Trigram+Bigram+Unigram+RegExp tagger                   : %5.2f%%' % (100.0 * tagger_accuracy(tagger4, test_tokens))
print 'Fourgram+Trigram+Bigram+Unigram+RegExp tagger          : %5.2f%%' % (100.0 * tagger_accuracy(tagger5, test_tokens))
print 'Fivegram+Fourgram+Trigram+Bigram+Unigram+RegExp tagger : %5.2f%%' % (100.0 * tagger_accuracy(tagger6, test_tokens))

